/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * (c) 2005 Jiri Benc <jbenc@suse.cz>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <linux/wireless.h>
#include "hostapd_ioctl.h"

#define IEEE80211_AUTH_ALG_OPEN		1
#define IEEE80211_AUTH_ALG_SHARED_KEY	2
#define IEEE80211_AUTH_ALG_LEAP		4

void usage()
{
	printf("Usage: dscape_set_algs [-oslL] device\n"
	       "Options:\n"
	       "  -o  open system\n"
	       "  -s  shared key\n"
	       "  -l  LEAP\n"
	       "  -L  list current algorithms\n");
	exit(1);
}

int main(int argc, char **argv)
{
	int opt;
	int list = 0;
	int parm[2] = { PRISM2_PARAM_AP_AUTH_ALGS, 0 };
	struct iwreq w;
	int hnd, res;

	for (;;) {
		opt = getopt(argc, argv, "oslL");
		if (opt < 0)
			break;
		if (opt == 'o')
			parm[1] |= IEEE80211_AUTH_ALG_OPEN;
		else if (opt == 's')
			parm[1] |= IEEE80211_AUTH_ALG_SHARED_KEY;
		else if (opt == 'l')
			parm[1] |= IEEE80211_AUTH_ALG_LEAP;
		else if (opt == 'L')
			list = 1;
		else
			usage();
	}
	if (argc <= optind)
		usage();
	strncpy(w.ifr_ifrn.ifrn_name, argv[optind], IFNAMSIZ);
	w.ifr_ifrn.ifrn_name[IFNAMSIZ - 1] = 0;
	memcpy(&w.u, parm, sizeof(parm));

	hnd = socket(PF_INET, SOCK_DGRAM, 0);
	if (hnd < 0) {
		printf("Error opening socket (%d)\n", hnd);
		return 1;
	}

	if (list) {
		res = ioctl(hnd, PRISM2_IOCTL_GET_PRISM2_PARAM, &w);
		if (res < 0) {
			printf("Error (%d)\n", res);
			return 1;
		}
		memcpy(parm, &w.u, sizeof(parm));
		printf("Configured algorithms:\n");
		if (parm[0] & IEEE80211_AUTH_ALG_OPEN)
			printf("open system\n");
		if (parm[0] & IEEE80211_AUTH_ALG_SHARED_KEY)
			printf("shared key\n");
		if (parm[0] & IEEE80211_AUTH_ALG_LEAP)
			printf("LEAP\n");
		if (parm[0] & ~(IEEE80211_AUTH_ALG_OPEN | IEEE80211_AUTH_ALG_SHARED_KEY | IEEE80211_AUTH_ALG_LEAP))
			printf("unknown (%x)\n", parm[0]);
	} else {
		res = ioctl(hnd, PRISM2_IOCTL_PRISM2_PARAM, &w);
		if (res < 0) {
			printf("Error (%d)\n", res);
			return 1;
		}
	}
	close(hnd);
	return 0;
}
